{% extends "basic_layout.html" %}
{# put the sidebar before the body #}
{% block sidebar2 %}{% endblock %}
{% block extrahead %}
{{ super() }}
{% endblock %}
{% set reldelim1 = ' &gt;' %}
{% block rootrellink %}
        <li><a href="{{ pathto('index') }}"><img src="{{ pathto('_static/fipy.png', 1) }}" alt="FiPy logo"
                 style="vertical-align: middle; margin-top: -1px"/>
        FiPy</a>{{ reldelim1 }}</li>
{% endblock %}
{% block header %}
{{ super() }}
{% endblock %}
{% block relbar1 %}
{{ super() }}
{% endblock %}
{% block relbar2 %}{% endblock %}
{% block content %}
  <div id="main" class="clearfix" role="main" tabindex="-1">
    <div class="main__inner">
      <div class="page-info">
        <a id="main-content"></a>
      </div>

      <div class="document">
    {%- block document %}
        <div class="documentwrapper">

      <div id="content" class="column">
      <div id="block-nist-www-content" class="nist-block">
        <section class="nist-page__content usa-section clearfix">

        {% block nistsoftwarebutton %} {% endblock %}

        <div class="grid-container margin-top-4">
        <div class="grid-row grid-gap-6">

  <div class="nist-page__region nist-page__region--content tablet-lg:grid-col-8">
              <div class="body" role="main">
		{% block body %} {% endblock %}
              {%- if show_copyright %}
                {%- if hasdoc('copyright') %}
                  {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
                {%- else %}
                  {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{% endtrans %}
                {%- endif %}
              {%- endif %}
                <div class='last-updated'>
              {%- if last_updated %}
                {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
              {%- endif %}
              {%- if show_sphinx %}
                {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx-doc.org/">Sphinx</a> {{ sphinx_version }}.{% endtrans %}
              {%- endif %}
                </div>
              </div>
  </div>  
  <aside class="nist-page__region nist-page__region--sidebar-second tablet-lg:grid-col-4">
  {%- block sidebar1 %}{{ sidebar() }}{% endblock %}
      <div class="clearer"></div>
  </aside>
        </div>
      </div>

      <div class="panel-region--postface">
      <div class="inner region-inner inside">
        
<div class="pane pane--nist-node-date-pane" >

    </div>
      </div>
    </div>

        </section>
        </div>
      </div>

      </div>
  {%- endblock %}  
    </div>          

    </div>

  </div>

</div>
{% endblock %}
{% block sidebarlogo %}
{{ super() }}
<strong>FiPy: A Finite Volume PDE Solver Using Python</strong><br />
Version {{ release }}
{% endblock %}
{% block footer %}
{% endblock %}
