<div class="sphinxsidebarsection" align="center">
<p />
<p class="logo">
<a href="{{ pathto('examples/cahnHilliard/generated/examples.cahnHilliard.mesh2D') }}"
><img alt="Illustration of phase separation due to the Cahn-Hilliard effect" 
title="Illustration of phase separation due to the Cahn-Hilliard effect" width="90%" src="{{ 
pathto('_static/examples/cahnHilliard/CH.png', 1) }}" class="logo"/></a><br />
<em>Cahn-Hilliard</em>
</p>
<p class="logo">
<a href="{{ pathto('examples/phase/generated/examples.phase.anisotropy') }}"
><img alt="Illustration of dendrite growth using the phase field method" 
title="Illustration of dendrite growth using the phase field method" width="90%" src="{{ 
pathto('_static/examples/phase/anisotropy/dendrite.png', 1) }}" 
class="logo"/></a><br />
<em>Phase Field</em>
</p>
<p class="logo">
<a href="{{ pathto('examples/levelSet/electroChem/README') }}"
><img alt="Illustration of electrochemical superfill using the level set method" 
title="Illustration of electrochemical superfill using the level set method" width="90%" src="{{ 
pathto('_static/examples/levelSet/electroChem/superfillImage.png', 1) }}" 
class="logo"/></a><br />
<em>Level Set</em>
</p>
</div>
