{#
    nist/localtoc.html
    ~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: local table of contents.

    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
    
    Modified for FiPy
#}
{%- if display_toc %}
  <div class="sphinxsidebarsection">
  <h3><a href="{{ pathto(master_doc) }}">{{ _('Table Of Contents') }}</a></h3>
  {{ toc }}
  </div>
{%- endif %}
