{#
    nist/searchbox.html
    ~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: quick search box.

    :copyright: Copyright 2007-2023 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.

    Modified for FiPy
#}
{%- if pagename != "search" and builder != "singlehtml" %}
<div class="sphinxsidebarsection">
<div id="searchbox" style="display: none" role="search">
  <div data-layout-content-preview-placeholder-label="&quot;Sphinx
  Search&quot; field" class="nist-block nist-block--search">
  <h2 id="searchlabel">{{ _('Quick search') }}</h2>
    <div class="searchformwrapper">
    <form class="search" action="{{ pathto('search') }}" method="get">
      <input type="text" name="q" aria-labelledby="searchlabel" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false"/>
      <input type="submit" value="{{ _('Go') }}" />
    </form>
    </div>
  </div>
</div>
<script>document.getElementById('searchbox').style.display = "block"</script>
</div>
{%- endif %}
