.ajax-progress{display:inline-block;padding:1px 5px 2px 5px;}[dir="rtl"] .ajax-progress{float:right;}.ajax-progress-throbber .throbber{display:inline;padding:1px 5px 2px;background:transparent url(/core/themes/stable/images/core/throbber-active.gif) no-repeat 0 center;}.ajax-progress-throbber .message{display:inline;padding:1px 5px 2px;}tr .ajax-progress-throbber .throbber{margin:0 2px;}.ajax-progress-bar{width:16em;}.ajax-progress-fullscreen{position:fixed;z-index:1000;top:48.5%;left:49%;width:24px;height:24px;padding:4px;opacity:0.9;border-radius:7px;background-color:#232323;background-image:url(/core/themes/stable/images/core/loading-small.gif);background-repeat:no-repeat;background-position:center center;}[dir="rtl"] .ajax-progress-fullscreen{right:49%;left:auto;}
.text-align-left{text-align:left;}.text-align-right{text-align:right;}.text-align-center{text-align:center;}.text-align-justify{text-align:justify;}.align-left{float:left;}.align-right{float:right;}.align-center{display:block;margin-right:auto;margin-left:auto;}
.js input.form-autocomplete{background-image:url(/core/themes/stable/images/core/throbber-inactive.png);background-repeat:no-repeat;background-position:100% center;}.js[dir="rtl"] input.form-autocomplete{background-position:0% center;}.js input.form-autocomplete.ui-autocomplete-loading{background-image:url(/core/themes/stable/images/core/throbber-active.gif);background-position:100% center;}.js[dir="rtl"] input.form-autocomplete.ui-autocomplete-loading{background-position:0% center;}
.fieldgroup{padding:0;border-width:0;}
.container-inline div,.container-inline label{display:inline;}.container-inline .details-wrapper{display:block;}
.clearfix:after{display:table;clear:both;content:"";}
.js details:not([open]) .details-wrapper{display:none;}
.hidden{display:none;}.visually-hidden{position:absolute !important;overflow:hidden;clip:rect(1px,1px,1px,1px);width:1px;height:1px;word-wrap:normal;}.visually-hidden.focusable:active,.visually-hidden.focusable:focus{position:static !important;overflow:visible;clip:auto;width:auto;height:auto;}.invisible{visibility:hidden;}
.item-list__comma-list,.item-list__comma-list li{display:inline;}.item-list__comma-list{margin:0;padding:0;}.item-list__comma-list li:after{content:", ";}.item-list__comma-list li:last-child:after{content:"";}
.js .js-hide{display:none;}.js-show{display:none;}.js .js-show{display:block;}
.nowrap{white-space:nowrap;}
.position-container{position:relative;}
.progress{position:relative;}.progress__track{min-width:100px;max-width:100%;height:16px;margin-top:5px;border:1px solid;background-color:#fff;}.progress__bar{width:3%;min-width:3%;max-width:100%;height:1.5em;background-color:#000;}.progress__description,.progress__percentage{overflow:hidden;margin-top:0.2em;color:#555;font-size:0.875em;}.progress__description{float:left;}[dir="rtl"] .progress__description{float:right;}.progress__percentage{float:right;}[dir="rtl"] .progress__percentage{float:left;}.progress--small .progress__track{height:7px;}.progress--small .progress__bar{height:7px;background-size:20px 20px;}
.reset-appearance{margin:0;padding:0;border:0 none;background:transparent;line-height:inherit;-webkit-appearance:none;-moz-appearance:none;appearance:none;}
.resize-none{resize:none;}.resize-vertical{min-height:2em;resize:vertical;}.resize-horizontal{max-width:100%;resize:horizontal;}.resize-both{max-width:100%;min-height:2em;resize:both;}
table.sticky-header{z-index:500;top:0;margin-top:0;background-color:#fff;}
.system-status-counter__status-icon{display:inline-block;width:25px;height:25px;vertical-align:middle;}.system-status-counter__status-icon:before{display:block;content:"";background-repeat:no-repeat;background-position:center 2px;background-size:20px;}.system-status-counter__status-icon--error:before{background-image:url(/core/themes/stable/images/core/icons/e32700/error.svg);}.system-status-counter__status-icon--warning:before{background-image:url(/core/themes/stable/images/core/icons/e29700/warning.svg);}.system-status-counter__status-icon--checked:before{background-image:url(/core/themes/stable/images/core/icons/73b355/check.svg);}
.system-status-report-counters__item{width:100%;margin-bottom:0.5em;padding:0.5em 0;text-align:center;white-space:nowrap;background-color:rgba(0,0,0,0.063);}@media screen and (min-width:60em){.system-status-report-counters{display:flex;flex-wrap:wrap;justify-content:space-between;}.system-status-report-counters__item--half-width{width:49%;}.system-status-report-counters__item--third-width{width:33%;}}
.system-status-general-info__item{margin-top:1em;padding:0 1em 1em;border:1px solid #ccc;}.system-status-general-info__item-title{border-bottom:1px solid #ccc;}
body.drag{cursor:move;}tr.region-title{font-weight:bold;}tr.region-message{color:#999;}tr.region-populated{display:none;}tr.add-new .tabledrag-changed{display:none;}.draggable a.tabledrag-handle{float:left;overflow:hidden;height:1.7em;margin-left:-1em;cursor:move;text-decoration:none;}[dir="rtl"] .draggable a.tabledrag-handle{float:right;margin-right:-1em;margin-left:0;}a.tabledrag-handle:hover{text-decoration:none;}a.tabledrag-handle .handle{width:14px;height:14px;margin:-0.4em 0.5em 0;padding:0.42em 0.5em;background:url(/core/themes/stable/images/core/icons/787878/move.svg) no-repeat 6px 7px;}a.tabledrag-handle:hover .handle,a.tabledrag-handle:focus .handle{background-image:url(/core/themes/stable/images/core/icons/000000/move.svg);}.touchevents .draggable td{padding:0 10px;}.touchevents .draggable .menu-item__link{display:inline-block;padding:10px 0;}.touchevents a.tabledrag-handle{width:40px;height:44px;}.touchevents a.tabledrag-handle .handle{height:21px;background-position:40% 19px;}[dir="rtl"] .touch a.tabledrag-handle .handle{background-position:right 40% top 19px;}.touchevents .draggable.drag a.tabledrag-handle .handle{background-position:50% -32px;}.tabledrag-toggle-weight-wrapper{text-align:right;}[dir="rtl"] .tabledrag-toggle-weight-wrapper{text-align:left;}.indentation{float:left;width:20px;height:1.7em;margin:-0.4em 0.2em -0.4em -0.4em;padding:0.42em 0 0.42em 0.6em;}[dir="rtl"] .indentation{float:right;margin:-0.4em -0.4em -0.4em 0.2em;padding:0.42em 0.6em 0.42em 0;}
.tablesort{display:inline-block;width:16px;height:16px;background-size:100%;}.tablesort--asc{background-image:url(/core/themes/stable/images/core/icons/787878/twistie-down.svg);}.tablesort--desc{background-image:url(/core/themes/stable/images/core/icons/787878/twistie-up.svg);}
div.tree-child{background:url(/core/themes/stable/images/core/tree.png) no-repeat 11px center;}div.tree-child-last{background:url(/core/themes/stable/images/core/tree-bottom.png) no-repeat 11px center;}[dir="rtl"] div.tree-child,[dir="rtl"] div.tree-child-last{background-position:-65px center;}div.tree-child-horizontal{background:url(/core/themes/stable/images/core/tree.png) no-repeat -11px center;}
.views-align-left{text-align:left;}.views-align-right{text-align:right;}.views-align-center{text-align:center;}.views-view-grid .views-col{float:left;}.views-view-grid .views-row{float:left;clear:both;width:100%;}.views-display-link + .views-display-link{margin-left:0.5em;}
.cke_reset_all .cke_dialog_body fieldset:not(.fieldgroup) > legend{position:relative;}
#backtotop{background:url(/modules/contrib/back_to_top/backtotop.png) no-repeat center center;bottom:20px;cursor:pointer;display:none;height:70px;position:fixed;right:20px;text-indent:-9999px;width:70px;z-index:300;}#backtotop:hover{opacity:0.8;}<!--[if lte IE 6]>
html{overflow-x:auto;overflow-y:hidden;}* html #backtotop{position:absolute;}<![endif]-->@media (-webkit-min-device-pixel-ratio:2),(min-resolution:192dpi){#backtotop{background:url(/modules/contrib/back_to_top/backtotop2x.png) no-repeat center center;background-size:70px 70px;}}
span.ext{background:url(/modules/contrib/extlink/extlink_s.png) 2px center no-repeat;width:10px;height:10px;padding-right:12px;text-decoration:none;}span.mailto{background:url(/modules/contrib/extlink/extlink_s.png) -20px center no-repeat;width:10px;height:10px;padding-right:12px;text-decoration:none;}@media print{span.ext,span.mailto{display:none;padding:0;}}
.social-media-sharing{display:block;}.social-media-sharing ul{padding-left:0px;display:inline-flex;text-align:center;}.social-media-sharing li{display:inline-block;margin-right:15px;list-style:none;list-style-type:none;}.social-media-sharing .share{cursor:pointer;}.social-media-sharing .share img{width:40px;}
.item-list .field--widget-term-reference-tree ul,.block .field--widget-term-reference-tree ul,.field--widget-term-reference-tree ul{list-style-type:none;margin-top:0;margin-bottom:0;margin-left:0;}.item-list .field--widget-term-reference-tree ul ul,.block .field--widget-term-reference-tree ul ul,.field--widget-term-reference-tree ul ul{margin-left:1.5em;}.item-list .field--widget-term-reference-tree ul .form-item,.block .field--widget-term-reference-tree ul .form-item,.field--widget-term-reference-tree ul .form-item{margin:0;padding:0;}.term-reference-tree-button,.no-term-reference-tree-button{width:16px;height:16px;display:inline-block;*display:inline;zoom:1;vertical-align:middle;margin-right:4px;}.term-reference-tree-button{background-image:url(/modules/contrib/term_reference_tree/images/bullet_toggle_minus.png);}.no-term-reference-tree-button{background-color:#ddd;}.term-reference-tree-button.term-reference-tree-collapsed{background-image:url(/modules/contrib/term_reference_tree/images/bullet_toggle_plus.png);}.field--widget-term-reference-tree .form-item{display:inline-block;*display:inline;zoom:1;}.field--widget-term-reference-tree .disabled{opacity:.5;}.field--widget-term-reference-tree .parent-term{display:inline-block;*display:inline;zoom:1;font-weight:bold;}.form-type-checkbox-tree .error{background-image:none;border:2px solid red;padding:3px;}.region-content ul.term-reference-tree-level,.term-reference-tree-level{padding:0;}.term-reference-tree-track-list li{list-style-type:none;list-style-image:none;margin-left:10px;padding-left:20px;min-height:16px;cursor:pointer;}.term-reference-tree-track-list li.track-item:hover{color:red;background-image:url(/modules/contrib/term_reference_tree/images/bullet_delete.png);background-repeat:no-repeat;background-position:middle left;}.term-reference-tree-track-list li.term_ref_tree_nothing_message{list-style-type:none;list-style-image:none;font-style:italic;cursor:default;}.term-reference-track-list-container{padding:5px;}.term-reference-track-list-label{font-weight:bold;}.term-tree-list .selected{font-weight:bold;}.term-tree-list .unselected{font-weight:normal;}.term-tree-list ul{margin-top:0;}.field--type-entity-reference ul.links .term-tree-list ul.term li{float:none;}
