a{text-decoration:underline;color:#000;word-wrap:break-word;}a:after{content:" (https://www.nist.gov" attr(href) ")";font-size:10pt;font-weight:400;}a[href^="http://"]:after,a[href^="https://"]:after{content:" (" attr(href) ")";}.nist-card__image a:after,.nist-featured__image a:after,.nist-hero__image a:after,.nist-image a:after,.nist-teaser__image a:after,a.nist-video-thumbnail__lightbox-trigger:after,a[href^="#"]:after,a[name]{display:none;}abbr[title]:after{content:" (" attr(title) ")";}.nist-print-header{display:block !important;margin-bottom:2em;}.nist-print-header__url{margin-bottom:2em;font-size:14px;}a#main-content{display:none;}.annual-nist-award__banner,.filters,.local-tasks,.nist-block--org,.nist-card__image,.nist-footer,.nist-image__lightbox-links,.nist-page__region--header-top,.nist-page__region--sidebar-first,.nist-page__region--sidebar-second-top,.nist-resource-block__icon,.nist-social__wrapper,.nist-toc,.nist-video,.nist-video__wrapper,.pager,.usa-banner,.usa-header{display:none !important;}@page{margin:.5in 1in;}body{font-family:Georgia,Times New Roman,Times,serif;font-size:16px;line-height:1.4;}*{text-align:left !important;color:#000 !important;}p{width:auto !important;}h1{font-size:24pt;}h2{font-size:18pt;margin-top:2em;}h3{margin-bottom:0;}.nist-image,.nist-image__wrapper{display:table;clear:both;outline:1px solid #000;margin:1em 0;padding:.5em;max-width:500px;border-collapse:collapse;}img{max-width:100%;height:auto;}.align-left.nist-image,.align-left.nist-image__wrapper{margin-right:1rem;}.align-right.nist-image,.align-right.nist-image__wrapper{margin-left:1rem;}.nist-image .nist-image__wrapper,.nist-image__wrapper .nist-image{outline:0;margin:0;}.nist-image__caption{font-style:italic;display:table-caption;caption-side:bottom;padding:4px;font-size:13px;}.nist-image__caption p{margin:0;}.nist-image__credit{font-size:10px;padding:0 4px 4px;}.nist-blog-header__hero img{max-width:500px;}.nist-teaser{margin-bottom:2em;}.nist-teaser__content-wrapper{overflow:hidden;}.nist-teaser:after{content:"";display:table;clear:both;}.nist-teaser__title{margin:0;}.nist-teaser__image{float:left;margin:0 1em 0 0;border:none;}.nist-teaser--grid .nist-teaser__image{width:200px;}.nist-hero{display:-webkit-box;display:-ms-flexbox;display:flex;margin:2rem -1rem;}.nist-hero__item{margin:0 1rem;}.nist-hero__caption p{margin:0;}.last-updated{font-style:italic;font-size:10pt;}.usa-accordion__content{display:block !important;}span.ext{display:none;}table{border-collapse:collapse;}table,td,th{border:1px solid #000;}td,th{padding:3px;text-align:left;}.nist-field--label-inline .nist-field__label{float:left;padding-right:.5em;font-weight:700;}.usa-accordion__heading .usa-accordion__button{border:0;background:none;padding:0;font-size:18pt;font-weight:700;}.nist-block__title{font-size:18pt;margin:2rem 0 1rem;font-weight:700;}.gray-box{border:1px solid #000;padding:1rem;margin-bottom:1rem;}.gray-box>*,.gray-box>.nist-block>h2{margin-top:0;}.nist-content-row:after{content:"";display:table;clear:both;}.nist-image-bg{background:none !important;color:#000 !important;}.paragraph--type--video-bg-color{display:none !important;}.nist-slider,.nist-slider .nist-slider__items{margin:0 !important;width:auto !important;}.nist-slider .nist-slider__items{-webkit-transform:none !important;transform:none !important;}.nist-slider .nist-slider__item{display:block !important;margin:0 !important;float:none !important;width:auto !important;}.nist-slider .nist-slider__controls,.nist-slider .nist-slider__direction-nav,.nist-slider .nist-slider__pauseplay{display:none !important;}.no-print{display:none;}
