/*
 * nist.css_t
 * ~~~~~~~~~~~~~~~
 *
 * NIST stylesheet -- nist theme.  Originally created by
 * Jonathan Guyer for FiPy.
 *
 */

@import url("basic.css");

@import url("css_xHLfTKufnRskE-WS-MHFRZw0xTl5z5FmoqUO5xrVbv0.css") all;
@import url("css_upsjsFTrh_luQ_mkVG8bvtY6hJP2qeBdVsxUinYrGlI.css") screen;
@import url("css_Wmk7cHp8tUe6LbVg3R63qxW01aH9Q-8Q4W2Hu6mxCwE.css") print;

/* -- page layout ----------------------------------------------------------- */

div.sphinxsidebar {
    width: 100%;
    margin: 0;
}

cite, code, tt {
    font-family: 'Consolas', 'Deja Vu Sans Mono',
                 'Bitstream Vera Sans Mono', monospace;
    font-size: 0.95em;
    letter-spacing: 0.01em;
}

tt {
    background-color: #f2f2f2;
    border-bottom: 1px solid #ddd;
    color: #333;
}

tt.descname, tt.descclassname, tt.xref {
    border: 0;
}

a tt {
    border: 0;
    color: #4070A0;
}

a tt:hover {
    color: #2491CF;
}

pre {
    font-family: 'Consolas', 'Deja Vu Sans Mono',
                 'Bitstream Vera Sans Mono', monospace;
    font-size: 0.95em;
    letter-spacing: 0.015em;
    line-height: 120%;
    padding: 0.5em;
    border: 1px solid #ccc;
    background-color: #f8f8f8;
}

pre a {
    color: inherit;
    text-decoration: underline;
}

div.admonition, div.warning {
    font-size: 0.9em;
    margin: 1em 0 1em 0;
    border: 1px solid #86989B;
    background-color: #f7f7f7;
    padding: 0;
}

div.admonition p, div.warning p {
    margin: 0.5em 1em 0.5em 1em;
    padding: 0;
}

div.admonition pre, div.warning pre {
    margin: 0.4em 1em 0.4em 1em;
}

div.admonition p.admonition-title,
div.warning p.admonition-title {
    margin: 0;
    padding: 0.1em 0 0.1em 0.5em;
    color: white;
    border-bottom: 1px solid #86989B;
    font-weight: bold;
    background-color: #406060;
}

div.admonition p.admonition-title a,
div.warning p.admonition-title a {
    color: white;
    text-decoration: underline;
}

div.warning {
    border: 1px solid #940000;
}

div.warning p.admonition-title {
    background-color: #CF0000;
    border-bottom-color: #940000;
}

div.admonition ul, div.admonition ol,
div.warning ul, div.warning ol {
    margin: 0.1em 0.5em 0.5em 3em;
    padding: 0;
}

div.sphinxsidebar #searchbox input[type="submit"] {
    width: auto;
}

.nist-block--contact .nist-block__title:before {
    background-image: url(user.svg);
}

.nist-block--mail .nist-block__title:before {
    background-image: url(envelope.svg);
}

.nist-block--org .nist-block__title:before {
    background-image: url(sitemap.svg);
}

div.sphinxsidebar ul ul {
    margin-left: 0;
}

html {
    font-family: Helvetica Neue,Helvetica,Roboto,Arial,sans-serif;
}

body {
    font-family: Helvetica Neue,Helvetica,Roboto,Arial,sans-serif;
}

body h1, h2, h3, h4, h5, h6 {
    font-family: Helvetica Neue,Helvetica,Roboto,Arial,sans-serif;
}

body h1 {
    margin-top: 0;
}

body h2 {
    font-size: 1.59706rem;
    line-height: 1.12707;
    border-bottom: 2px solid #005ea2;
    font-weight: 400;
    padding-bottom: .25rem;
    margin-top: 2.5rem;
    margin-bottom: .5rem;
    text-transform: uppercase;
    clear: both;
}

body h2.contentstablelink {
    border-bottom: none;
}

body h3 {
    font-size: 1.13125rem;
    line-height: 1.26796;
    font-weight: 700;
    padding: 0;
    margin-top: 1.25rem;
    margin-bottom: 0;
}

body h4 {
    margin-top: 1.25rem;
}

p.topless {
    margin-top: 0;
}

table.contentstable td, table.contentstable th {
    border: 0px;
}

h2.nist-block__title {
    margin-top: 0;
}

