"""Domain geometry and topology
"""
__docformat__ = 'restructuredtext'

from fipy.meshes.factoryMeshes import *
from fipy.meshes.periodicGrid1D import *
from fipy.meshes.periodicGrid2D import *
from fipy.meshes.periodicGrid3D import *
from fipy.meshes.skewedGrid2D import *
from fipy.meshes.tri2D import *
from fipy.meshes.gmshMesh import *
